﻿import re
from time import sleep
from openai import OpenAI
from datetime import datetime

client = OpenAI(
    base_url='https://api.deepseek.com',
    api_key='', #替换api_key
)

def translate_text(text_part):
    try:
        response = client.chat.completions.create(
            model='deepseek-chat',
            temperature=1.3,
            messages=[
                {
                    'role': 'system',
                    'content': '''
专业日语字幕翻译任务要求：
1. 翻译内容：精灵宝可梦系列作品的日语SRT字幕
2. 翻译标准：
   - 确保中文翻译流畅自然且语义准确
   - 专有名词统一使用最新官方译名（包括但不限于）：
     * 宝可梦名称
     * 人物角色名称
     * 地点名称
     * 道具物品名称
3. 格式规范：
   - 严格保留原始时间轴及其条数，禁止任何修改
   - 严格保留原始标点符号，禁止任何修改
4. 输出要求：
   - 仅输出翻译后的中文字幕内容
   - 禁止添加字幕以外的信息
   - 翻译后字幕时间轴条数保持一致
5. 注意事项：
   - 保持对话语气符合角色性格特征
   - 特殊招式名称需使用官方译法
   - 文化特定表达需恰当本地化
   - 可根据具体需求进一步补充专有名词对照表等细节信息
        '''
                },
                {
                    'role': 'user',
                    'content': text_part
                }
            ],
            stream=False
        )
        return response.choices[0].message.content
    except Exception as e:
        print(f"\n请求大模型异常{e}\n")
        return ""


def parse_time(time_str):
    """将SRT时间格式转换为毫秒"""
    time_format = "%H:%M:%S,%f"
    dt = datetime.strptime(time_str, time_format)
    return (dt.hour * 3600 + dt.minute * 60 + dt.second) * 1000 + dt.microsecond // 1000

def format_time(ms):
    """将毫秒转换为SRT时间格式"""
    hours = ms // 3600000
    ms %= 3600000
    minutes = ms // 60000
    ms %= 60000
    seconds = ms // 1000
    ms %= 1000
    return f"{hours:02d}:{minutes:02d}:{seconds:02d},{ms:03d}"

def load_srt(content):
    blocks = []
    pattern = re.compile(r'(\d+)\n(\d{2}:\d{2}:\d{2},\d{3}) --> (\d{2}:\d{2}:\d{2},\d{3})\n(.+?)(?=\n\d+\n|\Z)', re.DOTALL)
    for match in pattern.finditer(content):
        index = int(match.group(1))
        start = parse_time(match.group(2))
        end = parse_time(match.group(3))
        text = match.group(4).strip()
        blocks.append({'index': index, 'start': start, 'end': end, 'text': text})
    
    return blocks

def find_split_points(blocks, num_parts=5, min_gap=2000):
    """
    找到分割点，尽量等分同时避开时间轴过于接近的点
    min_gap: 最小分割间隔(毫秒)
    """
    total_duration = blocks[-1]['end'] - blocks[0]['start']
    part_duration = total_duration / num_parts
    
    split_points = []
    for i in range(1, num_parts):
        target_time = blocks[0]['start'] + i * part_duration
        
        # 找到最接近目标时间的字幕块
        best_block = None
        best_diff = float('inf')
        
        for j, block in enumerate(blocks):
            # 计算当前块开始时间与目标时间的差异
            diff = abs(block['start'] - target_time)
            
            # 检查与前一个分割点的间隔是否足够
            if split_points:
                prev_split_time = blocks[split_points[-1]]['end']
                gap = block['start'] - prev_split_time
                if gap < min_gap:
                    continue  # 跳过间隔太小的分割点
            
            if diff < best_diff:
                best_diff = diff
                best_block = j
        
        if best_block is not None:
            split_points.append(best_block)
    
    return split_points

def split_srt(blocks, split_points):
    """根据分割点将字幕块分成多个部分"""
    parts = []
    start_idx = 0
    
    for split_idx in split_points:
        parts.append(blocks[start_idx:split_idx])
        start_idx = split_idx
    
    parts.append(blocks[start_idx:])  # 添加最后一部分
    return parts

def get_srt_part(part):
    part_str = ''
    for i, block in enumerate(part, 1):
        part_str+=f"{block['index']}\n"
        part_str+=f"{format_time(block['start'])} --> {format_time(block['end'])}\n"
        part_str+=f"{block['text']}\n\n"
    return part_str

def save_subtitle(current_number):
    content = ''
    with open("C:\\Users\\iceunlim\\Desktop\\subtitle\\无印\\EP"+current_number+".srt", 'r', encoding='utf-8') as f:
        content = f.read()
    pattern = re.compile(r'<[^>]+>', re.S)

    blocks = load_srt(pattern.sub('', content)) # 正则去掉日版字幕中疑似html标签的信息
    
    # 找到分割点
    split_points = find_split_points(blocks, 6)
    
    # 分割字幕
    parts = split_srt(blocks, split_points)

    translated = ''
    for i, part in enumerate(parts, 1):
        max_retries = 3
        retry_count = 0
        success = False
        while retry_count < max_retries:
            print(f"\n第{current_number}集第{i}部分{len(part)}条数据,第{retry_count + 1}次翻译请求中\n")
            result = translate_text(get_srt_part(part))
            sleep(5)
            #print(f"\n大模型返回的翻译结果为\n{result}\n")
            lines = load_srt(result)
            print(f"\n原部分共{len(part)}条待翻译,翻译返回{len(lines)}条\n")
            if len(lines) == len(part):
                success = True
                translated += get_srt_part(lines)
                print(f"\n第{current_number}集第{i}部分,第{retry_count + 1}次翻译成功\n")
                break
            else:
                retry_count+=1
        if not success:
            print(f"\n第{current_number}集第{i}部分,3次重试后失败\n")
            raise ValueError

    with open("C:\\Users\\iceunlim\\Desktop\\subtitle\\翻译字幕\\无印\\EP"+current_number+".srt", "w", encoding="utf-8") as fw:
        fw.write(translated.strip())
    print(f"\n第{current_number}集翻译完成\n")

# 定义允许运行的时间段
start_time = datetime.strptime("00:30", "%H:%M").time()
end_time = datetime.strptime("08:00", "%H:%M").time()
for num in range(1,280):
    while True:
        now = datetime.now()
        current_time = now.time()
        if start_time <= current_time < end_time:
            try:
                save_subtitle(f"{num:03d}")
            except Exception as e:
                print(f"\n{num}处理失败：{e}\n")
            break  # 成功执行一次后跳出循环，继续下一个 num
        else:
            print(f"当前时间 {current_time.strftime('%H:%M')} 不在运行时间段，等待中...")
            sleep(60 * 10)  # 等待10分钟再检查一次